#pragma once

#include "history.hpp"
#include "samples.hpp"
#include "root-operation.hpp"
#include "control.hpp"
#include "record-operation.hpp"

struct LoopEngine {
  Samples samples;
  std::vector<Operation *> ops;
  History history;
  Control control;
  float in;
  float inputFadeTime;
  float outputFadeTime;

  LoopEngine(int samplesSize, int historySize, float inputFadeTime = 0.f, float outputFadeTime = 0.f) : samples(samplesSize), ops(historySize), history(ops), inputFadeTime(inputFadeTime), outputFadeTime(outputFadeTime) {
    history.ops[0] = new RootOp(&samples);
    control.process(); // prime the event processor
  }

  float process(float deltaTime, float input) {
    ControlEvent event = control.process();

    in = input;

    if (event == END) {
      history.end();
    }

    if (event == UNDO) {
      history.undo();
    }

    if (event == REDO) {
      history.redo();
    }

    if (event == RECORD) {
      Segment parentSegment = history.active()->getNextSegment();
      Segment segment = parentSegment.getNextSegment();
      history.push(new RecordOp(segment, &in, inputFadeTime, outputFadeTime));
    }

    float out = 0.f;
    for (int i = history.maxPos; i >= 0; --i) {
      if (history.ops[i]->shouldProcess()) {
        out += history.ops[i]->process(deltaTime);
      }
    }
    return out;
  }
};