#pragma once

#include "operation.hpp"

/**
 * Class: History
 *
 * A history maintains an ordered collection of operations.
 * A history should have one root operation at its start.
 */
struct History {
  int pos;
  int maxPos = pos;
  std::vector<Operation *> ops;
  // std::vector<std::unique_ptr<Operation>> ops; // 🐞 Safer to use unique_ptr

  History(std::vector<Operation *> ops) : ops(ops), pos(0) {}

  Operation *active() {
    return ops.at(pos);
  }

  void push(Operation *op) {
    active()->terminate();
    pos++;
    maxPos = pos;
    ops.at(pos) = op; // 🐞 Needs bounds check
  }

  void undo() {
    if (!canUndo()) {
      return;
    }
    active()->terminate();
    pos--;
    active()->recover();
  }

  void redo() {
    if (!canRedo()) {
      return;
    }
    active()->terminate();
    pos++;
    active()->recover();
  }

  void end() {
    active()->end();
  }

  bool canUndo() {
    return pos > 0;
  }

  bool canRedo() {
    return pos < maxPos;
  }
};