#pragma once

#include "operation.hpp"
#include "samples.hpp"
#include "asr-envelope.hpp"

class RecordOp : public Operation {
public:
  Segment recordSeg;
  AttackSustainRelease inputGate;
  AttackSustainRelease outputGate;

  RecordOp(Segment segment, float *input, float inputFadeTime, float outputFadeTime) : Operation(input), recordSeg(segment) {
    inputGate.setAttackReleaseTime(inputFadeTime, inputFadeTime);
    outputGate.setAttackReleaseTime(outputFadeTime, outputFadeTime);
  }

  float process(float deltaTime) {
    inputGate.process(deltaTime);

    recordSeg.next(inputGate.isOpen());

    if (inputGate.isOpen()) {
      recordSeg.write(*input);
    }

    if (inputGate.isClosing()) {
      recordSeg.write(recordSeg.read() * (1 - inputGate.getOutput()) + *input * inputGate.getOutput());
    }

    float out = recordSeg.read();

    return out * outputGate.process(deltaTime);
  }

  void end() {
    inputGate.release();
  }

  void terminate() {
    outputGate.release();
  }

  bool shouldProcess() {
    // Operation is still processing input
    if (!inputGate.isClosed()) {
      return true;
    }
    // Operation is still fading out
    if (!outputGate.isClosed()) {
      return true;
    }
    return false;
  }

  void recover() {
    // pos = startPos - 1;
    recordSeg.rewind();
    outputGate.reset();
  }

  Segment getNextSegment() {
    return recordSeg.getNextSegment();
  }
};
