#pragma once

#include "samples.hpp"

/**
 * Class: Segment
 *
 * A segment provides a way to read and write a specific segment of samples.
 * It grows dynamically but does not allocate sample memory itself.
 * A segment makes it easier to divide the total available samples into distinct groups.
 * A segment can produce another segment beginning where it ends, referencing the same sample memory.
 */
struct Segment {
  Samples *samples;
  int startPos;
  int endPos;
  int pos;

  Segment(Samples *samples, int startPos) : samples(samples), startPos(startPos), endPos(startPos), pos(startPos) {}

  void next(bool grow = false) {
    if (size() > 0) {
      pos++;
    }
    if (grow) {
      endPos++;
    }
    if (pos == endPos) {
      pos = startPos;
    }
  }

  float read() {
    return samples->read(pos);
  }

  void write(float input) {
    samples->write(pos, input);
  }

  int size() {
    return endPos - startPos;
  }

  void rewind() {
    pos = startPos - 1;
  }

  Segment getNextSegment() {
    return Segment{samples, endPos};
  }
};
